/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.activities;

import critiquer.activities.parameters.Parameter;

/**
 * 
 * Classes implementing this interface are checked before changes to activities
 * occur. If the methods in this interface return <code>true</code>, the
 * changes take place. Otherwise not.
 * 
 * @author olale
 * 
 */

public interface ActivityConsistencyCheck {
	/**
     * 
     * Whenever a paramChanged event occurs, this method is called in all
     * registered ActivityConsistencyCheck listeners that are available in an
     * activity.
     * 
     * @param <T> -
     *            The type of value the parameter represents
     * @param a
     * @param p
     * @param newValue -
     *            The suggested new value
     * @return true - if the change is allowed, false if not
     */
	public <T> boolean paramValueCheck(Activity a, Parameter<T> p, T newValue);

	public boolean childOfCheck(Activity parentActivity, Activity Child);
	public boolean orderingCheck(Activity beforeActivity,Activity AfterActivity);
	
}
