/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.activities;

import java.io.Serializable;

import critiquer.activities.parameters.Parameter;




/**
 * 
 * <p>
 * This interface is implemented by all constraints that listen to when
 * activities change, directly or indirectly. Also views are interested in
 * activity updates and implement this.
 * </p>
 * 
 * <p>
 * This interface extends Comparable in order to allow sorting of listeners so
 * that for instance the views are notified last of any updates.
 * </p>
 * 
 * @author olale
 * 
 */
public interface ActivityUpdateListener extends Serializable, Comparable<ActivityUpdateListener> {

	/**
     * Using this interface, the application lets the time panel know of changes
     * to the activities manipulated
     */

	/**
     * The activity has undergone major changes, refresh all information
     * dependent on it.
     */
	public void activityUpdated(Activity activity);

	/**
     * The activity was just removed
     */
	public void activityRemoved(Activity activity);

	/**
     * The activity has just been created
     */
	public void activityCreated(Activity activity);

	/**
     * A parameter has been added
     */
	public <T> void paramAdded(Activity activity, Parameter<T> p);

	/**
     * A parameter has changed value
     */
	public <T> void paramChanged(Activity activity, Parameter<T> p);

}
