/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.activities.parameters;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JPanel;

import critiquer.activities.Activity;
import critiquer.misc.ReferenceHolder;

/**
 * 
 * This parameter determines a commander who is responsible for carrying out one or more activities using one or more resources. 
 * All activities share a global list of commanders that is updated whenever new names are typed in the editable combo box. 
 * 
 * @author olale
 *
 */
public class CommanderParameter extends AbstractTaskViewParameter<JPanel, Commander> implements CommanderAddedListener {

	private transient JComboBox commanderList = null;
	private boolean enabled;	
	
	public CommanderParameter(String name, Activity activity) {
		super(name, activity);
		initComponent();
	}
	
	public boolean isEnabled() {
		return enabled && hasValue();
	}

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private void setCommanderEnabled(boolean enabled) {
		this.enabled = enabled;
		commanderList.setEnabled(enabled);
	}
	
	@Override
	public void initComponent() {
		component = new JPanel(new BorderLayout());
		component.setBorder(BorderFactory.createTitledBorder(getName()));
		initCommanderList();
		final JCheckBox agentAssignableChooser = new JCheckBox("Assign commander");
		agentAssignableChooser.setSelected(enabled);
		agentAssignableChooser.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				if (agentAssignableChooser.isSelected()) {
					setCommanderEnabled(true);
				} else {
					setCommanderEnabled(false);
				}
				valueChanged();
			}

			
			
		});
		component.add(agentAssignableChooser,BorderLayout.WEST);
		component.add(commanderList,BorderLayout.CENTER);
		
	}
	/**
	 * 
	 * Create the list component used to select commanders for activities.
	 *
	 */
	private void initCommanderList() {
		commanderList = new JComboBox(ReferenceHolder.commanders.getCommanders().keySet().toArray(new String[] {}));
		commanderList.setEnabled(enabled);
		commanderList.setEditable(true);
		if (hasValue()) {
			commanderList.setSelectedItem(getValue().getName());
		}
		commanderList.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				String commanderName = (String) commanderList.getSelectedItem();
				if (! ReferenceHolder.commanders.getCommanders().containsKey(commanderName)) {
					ReferenceHolder.commanders.addCommander(new Commander(commanderName));
					
				}
				if (hasValue()) {
					getValue().removeActivity(getActivity());
				}
				Commander commander = ReferenceHolder.commanders.getCommanders().get(commanderName);
				setValue(commander);
				commander.addActivity(getActivity());
			}
			
		});
	}

	/**
	 * @return Returns the commanderList.
	 */
	public JComboBox getCommanderList() {
		if (commanderList==null) {
			initCommanderList();
		}
		return commanderList;
	}

	/**
	 * 
	 * When the user has typed a new unique name of a commander, update the model used by the list component.
	 * 
	 * @see critiquer.activities.parameters.CommanderAddedListener#commanderAdded(critiquer.activities.parameters.Commander)
	 */
	public void commanderAdded(Commander commander) {
		commanderList.setModel(new DefaultComboBoxModel(ReferenceHolder.commanders.getCommanders().keySet().toArray(new String[] {})));
	}

	
}
