/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.activities.parameters;

import critiquer.activities.Activity;


public class LocationParameter extends ActivityParameter<Location> implements GeoViewParameter {

	
	public enum Type {
		/**
		 * Use this for locations that are the starting point of a transportation activity
		 */
		ORIG_LOC,
		/**
		 * Use this for the destination of a transportation activity
		 */
		DEST_LOC,
		/**
		 * Use this for locations that are not part of transportations
		 */
		STATIC_LOC;
	}

	
	/**
	 * 
	 */
	private static final long	serialVersionUID	= 1L;
	private Type type;
	
	
	public LocationParameter(Type type,String name,Activity a){
		super(name,a);
		this.type=type;
		value = new Location(this);
	}
	
	@Override
	public boolean setValue(Location newValue) {
		boolean consistent = Activity.isConsistent(getActivity(), this, value);
		//valueUpdate.acquire();
		if (consistent) {
			oldValue = value;
			value.removeParam(this);
			newValue.addParam(this);
			value = newValue;
			hasBeenSet=true;
			valueChanged();
		}
		return consistent;
	}
	
	@Override
	public String toString() {
		return name;
	}

	/**
	 * @return Returns the type.
	 */
	public Type getType() {
		return type;
	}

}
