/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.activities.parameters;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashSet;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import critiquer.activities.AbstractParamValueCheck;
import critiquer.activities.Activity;
import critiquer.activities.ActivityConsistencyCheck;
import critiquer.misc.ReferenceHolder;



public class NameParameter extends AbstractTaskViewParameter<JPanel,String> {

	/**
	 * 
	 */
	private static final long	serialVersionUID	= 8072678938385535268L;

	static {
		Activity.addStaticConsistencyChecker(new AbstractParamValueCheck() {

			protected boolean uniqueName(String text) {
				boolean ok=true;
				HashSet<Activity> activities = ReferenceHolder.allActivities;
				for (Activity activity : activities) {
					if (activity.toString().equals(text)) {
						ok=false;
						break;
					}
				}
				return ok;
			}
			
			public <T> boolean paramValueCheck(Activity a, Parameter<T> p, T newValue) {
				boolean consistent = true;
				if (p instanceof NameParameter) {
					NameParameter nameParam = (NameParameter) p;
					String newName = (String) newValue;
					consistent = uniqueName(newName);
					if (!consistent) {
						JOptionPane.showMessageDialog(nameParam.getComponent(),"Activity with that name already exists");
					}
				}
				return consistent;
			}
			
		});
	}
	
	public NameParameter(String name,Activity activity) {
		super(name,activity);
		this.value=activity.type+" "+activity.getNumActivities();
	}
	
	public NameParameter(Activity activity) {
		super("name",activity);
		this.value=activity.type+" "+activity.getNumActivities();
	}
	private transient JTextField nameField;
	
	@Override
	public String toString() {
		return value;
	}

	public void initComponent() {
		JPanel pane = new JPanel();
		pane.setBorder(BorderFactory.createTitledBorder("Name:"));
		nameField=new JTextField(this.value);
		selectText();
		nameField.addActionListener(new ActionListener(){

			public void actionPerformed(ActionEvent action) {
			
				boolean consistent = setValue(nameField.getText());
				if (!consistent) {
					nameField.setText(value);
				}
				selectText();
				
			}
			
		});
		pane.add(new JLabel("Edit activity name:"));
		pane.add(nameField);
		component=pane;
	}


	

	private void selectText() {
		nameField.setSelectionStart(0);
		nameField.setSelectionEnd(nameField.getText().length());
	}

}
