/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.activities.parameters;


/**
 * 
 * There may be parameters in activities that are nested within other parameters
 * but may have to be passed as reference to listeners that are interested in
 * knowing when things that affect activities are changed. Therefore we have
 * this interface which is implemented both by ActivityParameter and
 * AgentParameter. An AgentParameter is associated with a Commander, which in
 * turn can be associated with several activities. If agents are selected to
 * commanders, it affects all the activities that the commander is in control
 * of. Therefore such a change should be propagated to all concerned activities.
 * The only way to do that is to have a common interface for parameters that are
 * directly connected to activities and those that are nested deeper within. We
 * will try to keep the interface light therefore.
 * 
 * @author olale
 * 
 */
public interface Parameter<T> {

	public T getValue();

	public boolean hasValue();
}
