/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.activities.parameters;

import java.util.HashSet;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import critiquer.activities.Activity;



public class SelectionParameter<T> extends ActivityParameter<HashSet<T>> implements TaskViewParameter<JPanel, T> {

	/*
	 * Agentparameter and locationparameter should inherit from this one to have common functionality for
	 * selecting from a set of choices
	 * */
	
	/* (non-Javadoc)
	 * @see critiquer.ActivityParameter#getComponent()
	 */
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;



	public SelectionParameter(String name,Activity activity) {
		super(name,activity);
		value = new HashSet<T>();
	}

	private Vector<T> listObjects;
	protected JList list;
	private JPanel component;

	
	
	protected void createList(final Vector<T> listObjects) {
		final HashSet<T> selectedObjs = new HashSet<T>();
		component = new JPanel();
		component.setBorder(BorderFactory.createTitledBorder(toString()));
		
		list = new JList(listObjects);
		list.addListSelectionListener(new ListSelectionListener(){

			public void valueChanged(ListSelectionEvent e) {
				if (! e.getValueIsAdjusting()) {
					int[] selected = list.getSelectedIndices();
					selectedObjs.clear();
					for (int i = 0; i < selected.length; i++) {
						T elt = listObjects.get(selected[i]);
						selectedObjs.add(elt);
						
					}
					setValue(selectedObjs);
				}
				
			}
			
		});
		component.add(list);
		
	}
	
	/* (non-Javadoc)
	 * @see critiquer.activities.parameters.TaskViewParameter#getComponent()
	 */
	public JPanel getComponent() {
		if (component == null) {
			initComponent();
		}
		return component;
	}
	
	public void initComponent() {
		createList(listObjects);
	}



	@Override
	public String toString() {
		String s="";
		for (T selectedElement : getValue()) {
			s+=selectedElement.toString();
		}
		return s;
	}

	/**
	 * @return Returns the list.
	 */
	public JList getList() {
		return list;
	}

}
