/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.activities.parameters;

import java.io.Serializable;
import java.text.DateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;

import critiquer.activities.Activity;

/**
 * Parameter that holds a Calendar value. Used for start and end time of activities.
 * 
 * @author olale
 *
 */
public class TimeParameter extends ActivityParameter<Calendar>
		implements Serializable {

	public enum Type {
		
		START("start"),
		END("end");
		
		private String name;
		Type(String name) {
			this.name=name;
		}
		
		public String getName() {
			return name;
		}
	}
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private boolean editable;

	private Type type;

	public TimeParameter(String name, Type type, Activity a) {
		super(name, a);
		this.type=type;
		init();
	}

	private void init() {
		value = new GregorianCalendar();
		setEditable(true);

	}

	public int getHour() {
		return value.get(Calendar.HOUR_OF_DAY);
	}

	public int getMinute() {
		return value.get(Calendar.MINUTE);
	}

	@Override
	public String toString() {
		return getName()
				+ ": "
				+ DateFormat.getTimeInstance(DateFormat.SHORT).format(
						value.getTime());
	}

	/**
	 * @return Returns the editable.
	 */
	public boolean isEditable() {
		return editable;
	}

	/**
	 * @param editable
	 *            The editable to set.
	 */
	public void setEditable(boolean editable) {
		this.editable = editable;

	}

	public String getFullName() {
		return "[" + getActivity() + " " + getName() + "]";
	}

	/**
	 * @return Returns the type.
	 */
	public Type getType() {
		return type;
	}

}
