/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.communication;

import java.rmi.RemoteException;
import java.util.Date;
import java.util.HashSet;

import jxl.peermi.PeerRemoteObject;
import critiquer.activities.Activity;
import critiquer.activities.ActivityUtils;
import critiquer.resources.Agent;

/**
 * @author olale
 *
 */
public class ActivityProxy extends PeerRemoteObject implements RemoteActivity {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private Activity activity;
	private RemotePlan planner;

	/**
	 * @param activity
	 * @throws RemoteException 
	 */
	public ActivityProxy(Activity activity,RemotePlan planner) throws RemoteException {
		super();
		this.activity=activity;
		this.planner=planner;
	}

	/* (non-Javadoc)
	 * @see critiquer.communication.RemoteActivity#getStartTime()
	 */
	public Date getStartTime() throws RemoteException {
		return activity.getStartTime();
	}

	/* (non-Javadoc)
	 * @see critiquer.communication.RemoteActivity#getEndTime()
	 */
	public Date getEndTime() throws RemoteException {
		return activity.getEndTime();
	}

	/* (non-Javadoc)
	 * @see critiquer.communication.RemoteActivity#getAgents()
	 */
	public HashSet<RemoteAgent> getAgents() throws RemoteException {
		HashSet<Agent> agents = ActivityUtils.getAgentsInActivity(activity);
		HashSet<RemoteAgent> remoteAgents = new HashSet<RemoteAgent>();
		for (Agent agent : agents) {
			remoteAgents.add(new AgentProxy(agent));
		}
		return remoteAgents;
	}

	/* (non-Javadoc)
	 * @see critiquer.communication.RemoteActivity#describe()
	 */
	public String getName() throws RemoteException {
		// TODO Auto-generated method stub
		return activity.toString();
	}

	/* (non-Javadoc)
	 * @see critiquer.communication.RemoteActivity#getPlanner()
	 */
	public RemotePlan getPlanner() throws RemoteException {
		return planner;
	}

}
