/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.constraints;

import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.Calendar;

import org.apache.log4j.Logger;

import critiquer.gui.View;
import critiquer.simulation.AgentSimulationState;
import critiquer.simulation.DefaultAgentSimulationState;
import critiquer.simulation.SimulationEngine;


/**
 * @author olale
 *
 */
public class FuelLowCritic extends SimulationCritic implements TextCritic {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private SimulationEngine engine;
	private String information;

	/**
	 * 
	 */
	public FuelLowCritic() {
		super();
		// TODO Auto-generated constructor stub
	}


	/**
	 * 
	 * delegate to stepTo instead
	 * 
	 * @see critiquer.simulation.SimulationElement#stepForward(int)
	 */
	public void stepForward(int step) {
		Calendar nextTime = (Calendar) engine.getCurrentTime().clone();
		nextTime.add(Calendar.MINUTE, step);
		stepTo(nextTime);

	}

	/**
	 * 
	 * deletgate to stepTo
	 * 
	 * @see critiquer.simulation.SimulationElement#stepBackward(int)
	 */
	public void stepBackward(int step) {
		Calendar previousTime = (Calendar) engine.getCurrentTime().clone();
		previousTime.add(Calendar.MINUTE, step);
		stepTo(previousTime);
	}

	/**
	 * 
	 * check the fuel levels of the agent simulation states
	 * 
	 * @see critiquer.simulation.SimulationElement#stepTo(java.util.Calendar)
	 */
	public void stepTo(Calendar time) {
		ArrayList<AgentSimulationState> simulationStates = engine.getAgentSimulationStates();
		for (AgentSimulationState state : simulationStates) {
			Integer fuelLevel = (Integer) state.getProperty(DefaultAgentSimulationState.FUEL_LEVEL);
			logger.debug(state.getAgent()+" has fuel level "+fuelLevel);
			if (fuelLevel<=0) {
				information="Agent "+state.getAgent()+" has not enough fuel";
				setConsistent(false);
				return;
			}
		}
		setConsistent(true);
	}

	/* (non-Javadoc)
	 * @see critiquer.constraints.SingletonConstraint#getDescription()
	 */
	public String getDescription() {
		return "Displays the fuel status of\n"+
		"the various resources\n"+
		"during the mission and\n"+
		"reacts to low fuel levels";
	}

	/* (non-Javadoc)
	 * @see critiquer.gui.ViewRenderingListener#viewUpdated(critiquer.gui.View, java.awt.Graphics2D)
	 */
	public void viewUpdated(View v, Graphics2D g2) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see critiquer.simulation.SimulationElement#initSimulation(critiquer.simulation.SimulationEngine)
	 */
	public void initSimulation(SimulationEngine engine) {
		this.engine=engine;
		
	}

	/* (non-Javadoc)
	 * @see critiquer.constraints.TextCritic#getText()
	 */
	public String getText() {
		return information;
	}

}
