/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.constraints;

import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Point;
import java.awt.Stroke;
import java.util.ArrayList;
import java.util.HashSet;

import javax.swing.SwingUtilities;

import org.apache.log4j.Logger;

import critiquer.activities.Activity;
import critiquer.activities.ActivityUtils;
import critiquer.activities.parameters.AgentParameter;
import critiquer.activities.parameters.Parameter;
import critiquer.gui.ResourceView;
import critiquer.gui.View;
import critiquer.misc.Utils;
import critiquer.resources.Agent;

public class NonSuitableAgent extends StandardConstraint {

	/**
     * 
     */
	private static final long serialVersionUID = 1L;

	public <T> void paramChanged(Activity activity, Parameter<T> p) {

	}

	/*
     * (non-Javadoc)
     * 
     * @see critiquer.constraints.StandardConstraint#initApplicableViews()
     */
	@Override
	protected void initApplicableViews() {
		applicableViews.add(ResourceView.class);
	}

	public void viewUpdated(View v, Graphics2D g2) {
		if (v instanceof ResourceView) {
			ResourceView resourceView = (ResourceView) v;
			renderCritiqueResourceView(resourceView, g2);
		} else {
			Logger.getLogger(getClass()).error("This critic is inapplicable for view " + v.getClass().getSimpleName());
		}

	}

	private void renderCritiqueResourceView(ResourceView resourceView, Graphics2D g2) {
		Paint paint = g2.getPaint();
		Stroke stroke = g2.getStroke();
		g2.setStroke(new BasicStroke(4));

		ArrayList<Activity> selectedActivities = resourceView.getEvaluationActivities();
		if (selectedActivities != null) {
			for (Activity selectedActivity : selectedActivities) {
				AgentParameter parameter = ActivityUtils.getMainAgentParameter(selectedActivity);
				if (parameter != null) {
					Point activityLocation = Utils.getCenter(resourceView.getActivityArea(selectedActivity));
					HashSet<Agent> selectedAgents = ActivityUtils.getAgentsInActivity(selectedActivity);
					for (Agent agent : selectedAgents) {
						g2.setPaint(Agent.getSuitability(agent, selectedActivity).getPaint());
						Point agentLabelLocation = SwingUtilities.convertPoint(
								parameter.getAgentPanel(), 
								parameter.getAgentLocation(agent), 
								resourceView);
						g2.drawLine(activityLocation.x, activityLocation.y, agentLabelLocation.x, agentLabelLocation.y);
					}
				}
			}
		}
		g2.setPaint(paint);
		g2.setStroke(stroke);
	}

	public String getDescription() {
		String nl = System.getProperty("line.separator");
		return "Draws lines between selected agents and their" + nl
			   + " respective activities with a color depending"
			   + nl
			   + "on how well suited they are for the activity."
			   + nl
			   + "If they're well suited the color is green.";
	}

}
