/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.constraints;

import critiquer.activities.Activity;
import critiquer.activities.parameters.Parameter;
import critiquer.gui.GeoView;
import critiquer.gui.TimeView;
import critiquer.misc.ReferenceHolder;
import critiquer.simulation.SimulationElement;

/**
 * @author olale
 * 
 */
public abstract class SimulationCritic extends StandardConstraint implements SimulationElement {

	/**
     * @see critiquer.constraints.VisualConstraint#getApplicableViews()
     */
	public void initApplicableViews() {
		applicableViews.add(GeoView.class);
		applicableViews.add(TimeView.class);
	}

	/**
     * Restart the simulation when a new simulation critic was chosen.
     * 
     * @see critiquer.constraints.StandardConstraint#setActive(boolean)
     */
	@Override
	public void setActive(boolean active) {
		boolean wasActive = isActive();
		super.setActive(active);
		if (isActive() && !wasActive) {
			ReferenceHolder.timeView.initSimulation();
		}

	}

	/**
     * 
     * simulation critics don't care about parameter updates, only simulation
     * events. Parameters cannot change values during simulation anyway.
     * 
     * @see critiquer.activities.AbstractParamChangedListener#paramChanged(critiquer.activities.Activity,
     *      critiquer.activities.parameters.Parameter)
     */
	@Override
	public <T> void paramChanged(Activity activity, Parameter<T> p) {
		
	}

}
