/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.constraints;

import java.awt.Color;
import java.io.Serializable;
import java.util.List;

import critiquer.gui.View;
import critiquer.gui.ViewRenderingListener;


/**
 * Visual constraints can be rendered on top of the views and also have colors
 * that the user may modify.
 * 
 * Currently, every constraint has only one color associated with
 */

public interface VisualConstraint extends ViewRenderingListener, Serializable {

	public Color getColor();

	public void setColor(Color c);

	public List<Class<? extends View>> getApplicableViews();

	public boolean isApplicableFor(View view);
	
	/**
	 * Toggles the active state of this constraint on and off. This does not affect
	 * that updates that this constraint gets from the activities though
	 * 
	 * @param active
	 */
	public void setActive(boolean active);

	/**
	 * Should this constraint be displayed?
	 * 
	 * @return true iff this constraint should be displayed
	 */
	
	public boolean isActive();
}
