/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.constraints;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import critiquer.misc.Predicate;
import critiquer.misc.Utils;


/**
 * 
 * <p>A map of visual constraints, with flags indicating if they are active or not
 * </p> 
 * 
 * <p>We actually only care about the ViewRenderingListener constraints here,
 * the other ones can do their job without knowing about the view rendering
 * </p>  
 *  
 *  */

public class VisualConstraints {

	private static HashSet<VisualConstraint> constraints = new HashSet<VisualConstraint>();
	
	/**
	 * Toggles constraints on and off, though they are still active in the sense that they are
	 * updated with information from the activities
	 * */
	public static void setConstraintEnabled(Class constraintClass,boolean enabled){
		for (VisualConstraint c : constraints) {
			if (constraintClass.isInstance(c)) {
				c.setActive(enabled);
			}
		}
		
	}
	/**
	 * Retrieve a singleton constraint given its class
	 * 
	 * @param c the class of the constraint
	 * @return the singleton constraint
	 */
	public static VisualConstraint getConstraintByClass(final Class c) {
		return Utils.find(constraints, new Predicate<VisualConstraint>() {

			public boolean check(VisualConstraint constraint) {
				return c.isInstance(constraint);
			}
			
		});
	}
	
	public static void addConstraint(VisualConstraint c){
		constraints.add(c);	
	}
	
	public static void removeConstraint(VisualConstraint c) {
		constraints.remove(c);
	}
	/**
	 * 
	 * Get a list of the constraints that are currently displaying visual information.
	 * 
	 * @return a list of active constraints
	 */
	public static List<VisualConstraint> getActiveConstraints(){
		ArrayList<VisualConstraint> activeConstraints = new ArrayList<VisualConstraint>();
		for (VisualConstraint constraint : constraints) {
			if (constraint.isActive()) {
				activeConstraints.add(constraint);
			}
		}
		return activeConstraints;
	}

	
}
