/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.gui;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.LayoutManager;

import javax.swing.BorderFactory;
import javax.swing.JPanel;

import critiquer.constraints.VisualConstraint;
import critiquer.constraints.VisualConstraints;


public abstract class AbstractView extends JPanel implements View {

	/**
	 * Subclasses are free to place the critiquePanel wherever they like, but it
	 * is declared here for convenience. It is implcitly required by the View
	 * interface.
	 */
	protected ConstraintComponent	critiqueComponent;

	public JPanel getConstraintPanel() {
		return critiqueComponent.getPanel();
	}

	
	public AbstractView(LayoutManager layout) {
		super(layout);
		init();
	}

	public AbstractView() {
		super();
		init();
	}


	private void init() {
		critiqueComponent=new ConstraintComponent(this);
		critiqueComponent.getPanel().setBorder(BorderFactory.createEtchedBorder());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.JComponent#paint(java.awt.Graphics)
	 */
	@Override
	public void paint(Graphics g) {
		super.paint(g);
		updateView((Graphics2D) g);
	}

	public void updateView(Graphics2D g2) {
		for (VisualConstraint constraint : VisualConstraints.getActiveConstraints()) {
			if (constraint.isApplicableFor(this)) {				
				constraint.viewUpdated(this, g2);
			}
		}
	}

}
