/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.gui;

import java.awt.BorderLayout;
import java.awt.MenuBar;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTabbedPane;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import critiquer.communication.Communicator;
import critiquer.misc.ReferenceHolder;

import ch.randelshofer.quaqua.QuaquaManager;

public class ComPlan extends JFrame {

	/**
     * 
     */
	private static final long	 serialVersionUID = 1L;

	private JTabbedPane		   pane;

	private transient TimeView			  timePane;

	private transient ResourceView		  resourcePane;

	private transient CritiquePanel		 critiqueChooserPane;

	private transient OrganizationView	  orgPane;

	private transient GeoView			   gisPane;

	private transient static ComPlan   planningTool	 = null;

	private transient TaskView			  graphView;

	private transient StartupProgressWindow splashScreen;

	public static ComPlan getPlanningTool() {
		if (planningTool == null) {
			planningTool = new ComPlan();
		}
		return planningTool;
	}

	private ComPlan() {
		super("The ComPlan Planning Tool");
		ComPlan.planningTool = this;
		initialize();
	}

	private void initialize() {
		setLookAndFeel();
		if (ReferenceHolder.resourceDir == null) {
			selectResourceDirectory();
		}
		String name = JOptionPane.showInputDialog(this,"Enter your name:", "Input name", JOptionPane.PLAIN_MESSAGE);
		splashScreen = new StartupProgressWindow(20);
		splashScreen.setAlwaysOnTop(true);
		splashScreen.setVisible(true);
		splashScreen.newAction("Initializing remote interface");
		ReferenceHolder.communicator = new Communicator(name);
		splashScreen.actionDone();
		splashScreen.newAction("Creating organization model");
		ReferenceHolder.organizationModel.generateOrganization();
		splashScreen.actionDone();
		ReferenceHolder.topFrame = this;
		setTitle("ComPlan - "+name);
		pane = new JTabbedPane();
		critiqueChooserPane = new CritiquePanel(pane);
		createMenu(this);
		splashScreen.newAction("Initializing time view");
		timePane = new TimeView();
		splashScreen.actionDone();
		splashScreen.newAction("Initializing resource view");
		resourcePane = new ResourceView();
		splashScreen.actionDone();
		splashScreen.newAction("Initializing organization view");
		orgPane = new OrganizationView();
		splashScreen.actionDone();
		splashScreen.newAction("Initializing geographical view");
		gisPane = new GeoView();
		splashScreen.actionDone();
		splashScreen.newAction("Initializing task view");
		graphView = new TaskView();
		splashScreen.actionDone();
		pane.addTab("Task View", graphView);
		splashScreen.actionDone();
		pane.addTab("Organization view", orgPane);
		splashScreen.actionDone();
		pane.addTab("Resource View", resourcePane);
		splashScreen.actionDone();
		pane.addTab("Timeline View", timePane);
		splashScreen.actionDone();
		pane.addTab("Geographical view", gisPane);
		splashScreen.actionDone();
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(critiqueChooserPane, BorderLayout.WEST);
		getContentPane().add(pane, BorderLayout.CENTER);
		pack();
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		splashScreen.dispose();
	}

	private void setLookAndFeel() {
		try {
			String operatingSystem = System.getProperty("os.name");
			if (operatingSystem == "Mac OS X") {
				UIManager.setLookAndFeel(QuaquaManager.getLookAndFeelClassName());
			} else if (operatingSystem == "Linux") {
				UIManager.setLookAndFeel("com.sun.java.swing.plaf.gtk.GTKLookAndFeel");
			} else {
				UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			}
		} catch (ClassNotFoundException exception) {
			// TODO Auto-generated catch block
			exception.printStackTrace();
		} catch (InstantiationException exception) {
			// TODO Auto-generated catch block
			exception.printStackTrace();
		} catch (IllegalAccessException exception) {
			// TODO Auto-generated catch block
			exception.printStackTrace();
		} catch (UnsupportedLookAndFeelException exception) {
			Logger.getLogger(getClass()).error("Tried using unsupported Look and Feel.");
		}
	}

	private void createMenu(final JFrame parentFrame) {
		MenuBar menu = new MenuBar();
		parentFrame.setMenuBar(menu);
	}

	public static void main(String[] args) {
		processArguments(args);
		getPlanningTool().setVisible(true);

	}

	private static void selectResourceDirectory() {
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setMultiSelectionEnabled(false);
		fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		fileChooser.setDialogTitle("Select resource directory");
		if (fileChooser.showDialog(fileChooser, "Select") == JFileChooser.APPROVE_OPTION) {
			ReferenceHolder.resourceDir = fileChooser.getSelectedFile().getName();
		} else {
			JOptionPane.showMessageDialog(new JFrame(),
					"Invalid resource directory.\nMap will not be available",
					"Invalid directory",
					JOptionPane.WARNING_MESSAGE);
		}
	}

	private static void processArguments(String[] args) {
		String curDir = System.getProperty("user.dir");
		File resourceDir = new File(curDir,"resources");
		configureLogging(curDir);
		if (resourceDir.isDirectory()) {
			ReferenceHolder.resourceDir = resourceDir.getAbsolutePath();
		}
		for (int i = 0; i < args.length; i++) {
			if (args[i].equals("--resource-dir")) {
				ReferenceHolder.resourceDir = args[i + 1];
			}
		}
	}

	private static void configureLogging(String curDir) {
		String loggingPropertiesFileName = System.getProperty("log4j.configuration");
		boolean configured=false;
		if (loggingPropertiesFileName!=null) {
			File loggingPropertiesFile = new File(curDir, loggingPropertiesFileName);			
			if (loggingPropertiesFile.exists()) {
				PropertyConfigurator.configure(loggingPropertiesFileName);
			}
			configured=true;
		}
		
		if (!configured) {
			Logger.getLogger(ComPlan.class).error("No log4j property file found");
			BasicConfigurator.configure();
		}
	}

	public static void forceGUIUpdate() {
		if (planningTool != null) {
			planningTool.pane.revalidate();
			planningTool.pane.repaint();
		}
	}

	/**
     * @return Returns the splashScreen.
     */
	public StartupProgressWindow getSplashScreen() {
		return splashScreen;
	}
}
