/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.gui;

import java.awt.Color;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JCheckBox;

import critiquer.constraints.ConstraintStatusListener;
import critiquer.constraints.StandardConstraint;


class ConstraintCheckBox extends JCheckBox implements ItemListener, ConstraintStatusListener {

	/**
     * 
     */
	private static final long  serialVersionUID = 1L;

	private StandardConstraint c;

	private CheckBoxList	   parentList;

	private boolean			modelUpdate	  = false;

	public void itemStateChanged(ItemEvent e) {
		if (!modelUpdate) {
			if (isSelected()) {
				c.setActive(true);
			} else {
				c.setActive(false);
			}
		}
		modelUpdate = false;
		ComPlan.forceGUIUpdate();
	}

	public ConstraintCheckBox(StandardConstraint c, CheckBoxList chooserList) {
		this.c = c;
		this.parentList = chooserList;
		addItemListener(this);
		StandardConstraint.addStatusListener(this);

	}

	public Class getClassMember() {
		return c.getClass();
	}

	public String getText() {
		return (c != null ? c.getClass().getSimpleName() : "");
	}

	@SuppressWarnings("unchecked")
	public boolean isApplicableFor(View view) {
		return c.isApplicableFor(view);
	}

	/**
     * 
     * Set background color when activating the constraint. This method may be
     * called when the constraint has been deactivated from some other place, so
     * we need to ensure that we don't cause an infinite loop of signalling
     * update information. We use a flag modelUpdate to denote that we are
     * responding to an update that originated from the model of this view (the
     * constraint). When modelUpdate is true the itemStateChanged method should not
     * propagate back to the contraint again.
     * 
     * @see critiquer.constraints.ConstraintStatusListener#activeStatusChanged(critiquer.constraints.StandardConstraint)
     */
	public void activeStatusChanged(StandardConstraint constraint) {
		if (constraint == c) {
			updateBackground(constraint);
			if (c.isActive() != isSelected()) {
				modelUpdate = true;
				setSelected(c.isActive());
			}
		}
	}

	/**
     * 
     * Set the background color of this cell if it is inactive to indicate that
     * the constraint has information to show although it is not active.
     * 
     * @see critiquer.constraints.ConstraintStatusListener#violationStatusChanged(critiquer.constraints.StandardConstraint)
     */
	public void violationStatusChanged(StandardConstraint constraint) {
		if (constraint == c) {
			updateBackground(constraint);
		}
	}

	private void updateBackground(StandardConstraint constraint) {
		if (constraint.isConsistent()) {
			setBackground(Color.WHITE);
		} else {
			setBackground(Color.RED);
		}
		parentList.repaint();
	}

}