/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.gui;

import javax.swing.JPanel;

import critiquer.constraints.ConstraintStatusListener;
import critiquer.constraints.GUIComponentCritic;
import critiquer.constraints.StandardConstraint;


/**
 * @author olale
 *
 */
public class ConstraintComponent implements ConstraintStatusListener {

	private GUIComponentCritic controllingCritic;
	private JPanel panel = new JPanel();
	private View view;
	
	/**
	 * 
	 */
	public ConstraintComponent(View view) {
		this.view=view;
		StandardConstraint.addStatusListener(this);
	}
	
	public JPanel getPanel() {
		return panel;
	}
	
	/** 
	 * 
	 * Replace the controlling critic with this one if there is a new component critic active
	 * 
	 * @see critiquer.constraints.ConstraintStatusListener#activeStatusChanged(critiquer.constraints.StandardConstraint)
	 */
	public void activeStatusChanged(StandardConstraint c) {
		if (c.isActive() && c instanceof GUIComponentCritic && c!=controllingCritic && c.isApplicableFor(view)) {
			StandardConstraint oldCritic = (StandardConstraint)controllingCritic;
			if (oldCritic!=null) {
				oldCritic.setActive(false);
			}
			controllingCritic=(GUIComponentCritic) c;
			panel.add(controllingCritic.getComponent());
		} else if (c==controllingCritic && ! c.isActive()) {
			panel.removeAll();
			controllingCritic=null;
		}
	}

	/** 
	 * Nothing to do, the critic should present all information necessary.
	 * 
	 * @see critiquer.constraints.ConstraintStatusListener#violationStatusChanged(critiquer.constraints.StandardConstraint)
	 */
	public void violationStatusChanged(StandardConstraint c) {
		
	}
}
