/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import critiquer.constraints.ConstraintStatusListener;
import critiquer.constraints.StandardConstraint;
import critiquer.constraints.TextCritic;


/**
 * 
 * This class models the critic text area and makes cure to clear
 * @author olale
 *
 */
public class CriticTextArea implements ConstraintStatusListener {

	private JTextArea textArea = new JTextArea();
	private TextCritic controllingCritic;
	private JScrollPane areaScrollPane;
	/**
	 * 
	 */
	public CriticTextArea() {
		super();
		StandardConstraint.addStatusListener(this);
		textArea.setFont(new Font("Garamond", Font.TRUETYPE_FONT, 16));
		
        textArea.setLineWrap(true);
        textArea.setWrapStyleWord(true);
        textArea.setEnabled(false);
        areaScrollPane = new JScrollPane(textArea);
        areaScrollPane.setVerticalScrollBarPolicy(
                        JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        areaScrollPane.setPreferredSize(new Dimension(250, 400));
        areaScrollPane.setBorder(
            BorderFactory.createCompoundBorder(
                BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder("Critic text"),
                                BorderFactory.createEmptyBorder(5,5,5,5)),
                areaScrollPane.getBorder()));
	}

	public JComponent getComponent() {
		return areaScrollPane;
	}
	/** 
	 * 
	 * Replace the controlling critic with the argument critic if there is a new text critic active
	 * 
	 * @see critiquer.constraints.ConstraintStatusListener#activeStatusChanged(critiquer.constraints.StandardConstraint)
	 */
	public void activeStatusChanged(StandardConstraint c) {
		if (c.isActive() && c instanceof TextCritic) {
			/**
			 * Activation or re-activation of a critic
			 */
			if (c!=controllingCritic) {
				/**
				 * De-activate the old text critic
				 */
				if (controllingCritic!=null) {
					StandardConstraint oldCritic = (StandardConstraint) controllingCritic;
					oldCritic.setActive(false);
				}
				controllingCritic = (TextCritic) c;	
			}
			if (!c.isConsistent()) {				
				textArea.setText(controllingCritic.getText());
			} else {
				textArea.setText("Displaying information from critic "+c.getClass().getSimpleName()+"\n\n"+
						"Description: "+c.getDescription());
			}
		} else if (!c.isActive() && c==controllingCritic) {
			/**
			 * De-activation of a critic
			 */
			textArea.setText("");
		}
	}

	/** 
	 * Render critique if it is our constraint that fired this and it is active
	 * 
	 * @see critiquer.constraints.ConstraintStatusListener#violationStatusChanged(critiquer.constraints.StandardConstraint)
	 */
	public void violationStatusChanged(StandardConstraint c) {
		if ( c==controllingCritic && c.isActive() )  {
			if (!c.isConsistent()) {				
				textArea.setText(controllingCritic.getText());
			} else {
				textArea.setText("");
			}
		}
	}
}
