/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.util.ArrayList;
import java.util.Vector;

import javax.swing.DefaultListModel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import critiquer.constraints.ConstraintFactory;
import critiquer.constraints.SingletonConstraint;
import critiquer.constraints.StandardConstraint;
import critiquer.misc.ReferenceHolder;


/**
 * 
 * <p>
 * A panel that displays a list of visual constraints that may be enabled on a
 * per-view basis. Each constraint may be applicable to one or more views, and
 * only those applicable to the current view are displayed as options in the
 * pane.
 * </p>
 * 
 * <p>
 * All constraints represent singleton constraint objects listed in the
 * VisualConstraints class.
 * </p>
 * 
 * @author olale
 * 
 */
public class CritiquePanel extends JPanel {

	/**
     * 
     */
	private static final long serialVersionUID = 1L;

	private Vector<ConstraintCheckBox> createCheckboxArray(ArrayList<Class<? extends StandardConstraint>> constraintClasses,
														   CheckBoxList chooserList) {
		Vector<ConstraintCheckBox> checkboxes = new Vector<ConstraintCheckBox>();
		ConstraintCheckBox checkbox = null;
		for (Class constraintClass : constraintClasses) {
			SingletonConstraint singletonConstraint = ConstraintFactory.getSingletonConstraint(constraintClass);
			if (singletonConstraint instanceof StandardConstraint) {
				StandardConstraint standardConstraint = (StandardConstraint) singletonConstraint;
				checkbox = new ConstraintCheckBox(standardConstraint, chooserList);
				checkbox.setToolTipText(standardConstraint.getDescription());

				checkboxes.add(checkbox);
			}
		}
		return checkboxes;
	}

	private CheckBoxList			   chooserList;

	private CriticTextArea			 infoArea = new CriticTextArea();

	private DefaultListModel		   chooserListModel;

	private Vector<ConstraintCheckBox> checkBoxes;

	/**
     * User the tabbed pane as an argument to select which critics should be
     * selectable based on the current view
     */
	public CritiquePanel(final JTabbedPane pane) {
		super(new BorderLayout());
		chooserListModel = new DefaultListModel();
		chooserList = new CheckBoxList(chooserListModel);
		checkBoxes = createCheckboxArray(ReferenceHolder.constraintClasses,chooserList);
		pane.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				Component component = pane.getSelectedComponent();
				if (component instanceof View) {
					View currentView = (View) component;
					chooserListModel.removeAllElements();
					for (ConstraintCheckBox box : checkBoxes) {

						if (box.isApplicableFor(currentView)) {
							chooserListModel.addElement(box);

						}

					}
				}
			}

		});
		ReferenceHolder.critiqueInfoArea = infoArea;

		add(new JScrollPane(chooserList), BorderLayout.NORTH);
		add(infoArea.getComponent(), BorderLayout.CENTER);
	}

}
