/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.gui;

import java.awt.BorderLayout;
import java.awt.Shape;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Vector;

import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import critiquer.activities.Activity;
import critiquer.activities.ActivityUpdateListener;
import critiquer.activities.ActivityUtils;
import critiquer.activities.parameters.Location;
import critiquer.activities.parameters.LocationParameter;
import critiquer.activities.parameters.NameParameter;
import critiquer.activities.parameters.Parameter;
import critiquer.misc.ReferenceHolder;
import critiquer.misc.Utils;



public class GeoView extends AbstractView {

	/**
	 * 
	 */
	private static final long	serialVersionUID	= 1L;
	private ScrollableMap	 scrollableMap;
	private JScrollPane	mapPane;
	private DefaultComboBoxModel locationModel;
	private transient JComboBox locationList;
	
	/**
	 * Metres per pixel
	 */
	private double scale = 150;
	private ActivityUpdateListener locationModelUpdater = new ActivityUpdateListener(){

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		public void activityCreated(Activity activity) {
			//updateLocationModel();
		}

		public void activityRemoved(Activity a) {
			updateLocationModel();
		}

		public void activityUpdated(Activity activity) {
			updateLocationModel();
		}

		public <T> void paramAdded(Activity activity, Parameter<T> p) {
			updateLocationModel();
		}

		public <T> void paramChanged(Activity activity, Parameter<T> p) {
			if (p instanceof NameParameter || p instanceof LocationParameter) {
				updateLocationModel();
			}
		}

		public int compareTo(ActivityUpdateListener o) {
			return 0;
		}
		
	};
	

	public GeoView() {
		super();
		setLayout(new BorderLayout());
		init();
		ReferenceHolder.simulationView=this;
	}
	
	private synchronized void updateLocationModel() {
		locationModel.removeAllElements();
		HashSet<Location> locations = new HashSet<Location>();
		HashSet<Activity> activities = ReferenceHolder.allActivities;
		for (Activity activity : activities) {
			ArrayList<LocationParameter> locationParams = ActivityUtils.filterParams(activity, LocationParameter.class);
			for (LocationParameter parameter : locationParams) {
				locations.add(parameter.getValue());
			}
		}
		List<Location> locList = sortLocations(locations);
		for (Location location : locList) {
			locationModel.addElement(location);
		}
		scrollableMap.repaint();
	}


	private List<Location> sortLocations(Collection<Location> locations) {
		Vector<Location> locList = Utils.toVector(locations);
		Collections.sort(locList, new Comparator<Location>() {

			public int compare(Location o1, Location o2) {
				int comp=0;
				if (o1.getArea()==null && o2.getArea()!=null) {
					comp=-1;
				} else if (o2.getArea()==null && o1.getArea()!=null) {
					comp=1;
				}
				return comp;
			}
			
		});
		return locList;
	}
	
	public void init(){
		scrollableMap = new ScrollableMap(	new ImageIcon(ReferenceHolder.resourceDir + System.getProperty("file.separator")
		                                  	              + "karta.jpg"),
		                                  	              10,
		                                  	              this);
		mapPane = new JScrollPane(scrollableMap);
		add(mapPane,BorderLayout.CENTER);
		JPanel locationParameterPanel = new JPanel(new BorderLayout());
		locationModel = new DefaultComboBoxModel();
		locationList = new JComboBox(locationModel);
		locationList.setEditable(false);
		locationList.addActionListener(new ActionListener(){

			public void actionPerformed(ActionEvent e) {
				
					scrollableMap.setSelectedLocation((Location) locationList
						.getSelectedItem());
					scrollableMap.repaint();
							
			}
			
		});

		Activity.addActivityUpdateListener(locationModelUpdater);
		locationParameterPanel.add(new JLabel("Select a location"),
				BorderLayout.WEST);
		locationParameterPanel.add(locationList,BorderLayout.CENTER);
		locationParameterPanel.add(critiqueComponent.getPanel(),BorderLayout.SOUTH);
		
		add(locationParameterPanel,BorderLayout.SOUTH);
	}
	
	
	public Shape getActivityArea(Activity activity) {
		// TODO Auto-generated method stub
		return null;
	}

	public ArrayList<Activity> getEvaluationActivities() {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * @return Returns the scale of this map in meters per pixel.
	 */
	public double getScale() {
		return scale;
	}

	/**
	 * @param scale Set the scale (meters per pixel)
	 */
	public void setScale(double scale) {
		this.scale = scale;
	}

	/**
	 * @return Returns the locationModel.
	 */
	public DefaultComboBoxModel getLocationModel() {
		return locationModel;
	}

}
