/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.gui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.HeadlessException;
import java.awt.Menu;
import java.awt.MenuBar;
import java.awt.MenuItem;
import java.awt.MenuShortcut;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectInputStream;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;
import java.util.List;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.apache.log4j.Logger;
import org.tigris.gef.base.LayerManager;
import org.tigris.gef.base.LayerPerspective;
import org.tigris.gef.event.GraphSelectionEvent;
import org.tigris.gef.event.GraphSelectionListener;
import org.tigris.gef.graph.GraphModel;
import org.tigris.gef.graph.presentation.DefaultGraphModel;
import org.tigris.gef.graph.presentation.JGraph;
import org.tigris.gef.graph.presentation.NetEdge;
import org.tigris.gef.graph.presentation.NetNode;

import critiquer.activities.Activity;
import critiquer.activities.ActivityUtils;
import critiquer.activities.parameters.AbstractTaskViewParameter;
import critiquer.activities.parameters.ActivityParameter;
import critiquer.activities.parameters.Commanders;
import critiquer.activities.parameters.TaskViewParameter;
import critiquer.gui.graph.ActivityEditor;
import critiquer.gui.graph.ActivityFigNode;
import critiquer.gui.graph.ActivityNode;
import critiquer.gui.graph.GraphRenderingListener;
import critiquer.gui.graph.InheritanceEdge;
import critiquer.gui.graph.InheritanceFigEdge;
import critiquer.gui.graph.OrderingEdge;
import critiquer.gui.graph.OrderingFigEdge;
import critiquer.misc.ReferenceHolder;
import critiquer.misc.Utils;
import critiquer.resources.Agent;



public class TaskView extends AbstractView implements GraphRenderingListener {

	/**
     * 
     */
	private static final long serialVersionUID = 1L;

	private final class ResetItem implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if (graph.getEditor().getLayerManager().getActiveLayer() instanceof LayerPerspective) {
				DefaultGraphModel model = (DefaultGraphModel) graph.getGraphModel();
				deleteNodes(model);
				deleteEdges(model);

			} else {
				logger.error("No active LayerPerspective");
			}
		}
	}

	private final class OpenItem implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			FileDialog fileDialog = new FileDialog(mainWindow, "Open plan", FileDialog.LOAD);
			fileDialog.setVisible(true);

			if (fileDialog.getFile() != null) {
				String fileName = fileDialog.getDirectory() + System.getProperty("file.separator") + fileDialog.getFile();
				File file = new File(fileName);
				try {
					ObjectInput is = new ObjectInputStream(new FileInputStream(file));

					ReferenceHolder.commanders = (Commanders) is.readObject();
					DefaultGraphModel model = (DefaultGraphModel) is.readObject();

					DefaultGraphModel oldModel = (DefaultGraphModel) graph.getGraphModel();
					/**
                     * Clear the old model and replace it with a new one by
                     * hand..
                     */
					deleteNodes(oldModel);
					deleteEdges(oldModel);
					addNodes(model, oldModel);
					addEdges(model, oldModel);
					ReferenceHolder.organizationModel.setRootAgent((Agent) is.readObject());

				} catch (Exception exception) {
					exception.printStackTrace();
				}
			}

		}
	}

	private final class SaveItem implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			FileDialog fileDialog = new FileDialog(mainWindow, "Save plan", FileDialog.SAVE);
			fileDialog.setVisible(true);
			String fileName = fileDialog.getDirectory() + System.getProperty("file.separator") + fileDialog.getFile();
			if (fileName != null) {

				try {

					File file = new File(fileName);
					boolean canCreate = file.createNewFile();
					if (!canCreate) {
						logger.error("Couldn't create file " + fileName);
					}
					ObjectOutput os = new ObjectOutputStream(new FileOutputStream(file));
					/**
                     * Write the commanders first so they are available when the
                     * activities are loaded back
                     */
					os.writeObject(ReferenceHolder.commanders);

					DefaultGraphModel model = (DefaultGraphModel) graph.getGraphModel();
					List activityNodes = model.getNodes();
					for (Object object : activityNodes) {
						if (object instanceof ActivityNode) {
							ActivityNode activityNode = (ActivityNode) object;
							activityNode.preSave();
						}
					}
					os.writeObject(model);
					os.writeObject(ReferenceHolder.organizationModel.getRoot());

				} catch (FileNotFoundException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
		}
	}

	private static class ActivityAddItem extends MenuItem implements ActionListener {

		/**
         * 
         */
		private static final long serialVersionUID  = 1L;

		/**
         * This is used to create subsequent keyboard shortcut entries
         */
		private static int[]	  keyboardShortcuts = new int[] { KeyEvent.VK_1,
															KeyEvent.VK_2,
															KeyEvent.VK_3,
															KeyEvent.VK_4,
															KeyEvent.VK_5,
															KeyEvent.VK_6,
															KeyEvent.VK_7,
															KeyEvent.VK_8,
															KeyEvent.VK_9 };

		private static int		index			 = 0;

		private Class			 activityClass;

		private TaskView		  graphView;

		public ActivityAddItem(TaskView g, Class activityClass, String label) throws HeadlessException {
			super(label);
			this.graphView = g;
			this.activityClass = activityClass;
			addActionListener(this);
			if (index < 10) {
				setShortcut(new MenuShortcut(keyboardShortcuts[index++]));
			} // We could add Shift and Alt-modifiers to increase the list of
			// shortcuts if necessary

		}

		public void actionPerformed(ActionEvent e) {
			graphView.createNode(activityClass);
		}

	}

	private JGraph	graph;

	private JFrame	mainWindow;

	private Container pane;

	private JPanel	activityPane;

	private JLabel	defaultLabel;

	protected Vector  selection = new Vector();

	private JPanel	southPane;

	private transient Logger logger = Logger.getLogger(getClass());

	/**
     * This constructor is to be used from within the planning tool
     */

	public TaskView() {
		setLayout(new BorderLayout());
		ActivityEditor activityEditor = new ActivityEditor(ReferenceHolder.getGraphModel(), null);
		activityEditor.addRenderingListener(this);
		this.graph = new JGraph(activityEditor);
		// graph.getEditor().setGridHidden(true);

		defaultLabel = new JLabel("Select an activity to display its parameters");
		GraphSelectionListener activityPaneUpdater = new GraphSelectionListener() {

			/**
             * Update the activity panel if there is one node selected and it is
             * an activity fig node
             */
			public void selectionChanged(GraphSelectionEvent gse) {
				selection = gse.getSelections();
				activityPane.removeAll();
				if (selection.size() == 1 && selection.get(0) instanceof ActivityFigNode) {
					ActivityFigNode activityFigNode = (ActivityFigNode) selection.get(0);
					activityPane.add(getActivityPanel(activityFigNode.getActivityNode().getActivity()));

				} else {
					activityPane.add(defaultLabel);
				}
				activityPane.revalidate();
				activityPane.repaint();
			}

		};

		TaskView.addGraphSelectionListener(graph, activityPaneUpdater);
		Activity.addNameChangeListener(new NameChangeListener() {

			public void nameChanged() {
				graph.revalidate();
				graph.repaint();
			}

		});
		this.activityPane = new JPanel(new GridLayout(0, 1));

		Rectangle bounds = new Rectangle(10, 10, 500, 500);
		LayerManager lm = graph.getEditor().getLayerManager();
		Dimension preferredSize = new Dimension(500, 500);
		graph.setDrawingSize(preferredSize);
		graph.setPreferredSize(preferredSize);
		LayerPerspective perspective = (LayerPerspective) lm.getActiveLayer();
		perspective.addNodeTypeRegion(ActivityNode.class, bounds);

		mainWindow = ReferenceHolder.topFrame;
		addFileMenu();
		addActivityMenu();
		addEditMenu();
		add(graph, BorderLayout.CENTER);
		southPane = new JPanel(new BorderLayout());
		southPane.add(activityPane, BorderLayout.CENTER);
		southPane.add(critiqueComponent.getPanel(), BorderLayout.SOUTH);

		add(southPane, BorderLayout.SOUTH);
	}

	private void addEditMenu() {
		Menu editMenu = new Menu("Edit");
		mainWindow.getMenuBar().add(editMenu);
		MenuItem deleteItem = new MenuItem("Delete selected");
		editMenu.add(deleteItem);
		deleteItem.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				if (selection.size() == 1) {
					// There is something to remove..
					Object elt = selection.get(0);
					if (elt instanceof OrderingFigEdge) {
						OrderingFigEdge figEdge = (OrderingFigEdge) elt;
						OrderingEdge edge = (OrderingEdge) figEdge.getOwner();

						edge.deleteFromModel();
					} else if (elt instanceof InheritanceFigEdge) {
						InheritanceFigEdge figEdge = (InheritanceFigEdge) elt;
						InheritanceEdge edge = (InheritanceEdge) figEdge.getOwner();

						edge.deleteFromModel();
					} else if (elt instanceof ActivityFigNode) {
						ActivityFigNode figNode = (ActivityFigNode) elt;
						ActivityNode activityNode = (ActivityNode) figNode.getOwner();
						activityNode.deleteFromModel();
					} else {
						logger.error("Currently unsupported operation to remove " + elt.getClass()
																							 .getSimpleName());
					}

				}
			}

		});
		deleteItem.setShortcut(new MenuShortcut(KeyEvent.VK_DELETE, false));
	}

	/**
     * Use this constructor for stand-alone tests of the graph view
     */

	public TaskView(JFrame mainWindow) {
		this.graph = new JGraph();
		graph.setBounds(new Rectangle(10, 10, 300, 300));
		LayerManager lm = graph.getEditor().getLayerManager();

		LayerPerspective perspective = (LayerPerspective) lm.getActiveLayer();
		perspective.addNodeTypeRegion(ActivityNode.class, new Rectangle(10, 10, 300, 300));

		this.mainWindow = mainWindow;
		initStandaloneMenu();
		pane = mainWindow.getContentPane();
		pane.add(graph, BorderLayout.CENTER);
		mainWindow.setSize(300, 300);
		mainWindow.setVisible(true);
	}

	public static void addGraphSelectionListener(JGraph graph, GraphSelectionListener l) {
		graph.getEditor().getSelectionManager().addGraphSelectionListener(l);
	}

	private void initStandaloneMenu() {
		MenuBar mb = new MenuBar();
		mainWindow.setMenuBar(mb);
		addActivityMenu();
		addFileMenu();

	}

	/**
     * 
     * This is used instead of DefaultGraphModel.removeAll which does not seem
     * to behave as expected.
     * 
     */
	private void deleteNodes(DefaultGraphModel oldModel) {
		List nodes = oldModel.getNodes();
		int size = nodes.size();
		for (int i = 0; i < size; i++) {
			Object object = nodes.get(0);
			if (object instanceof NetNode) {
				NetNode node = (NetNode) object;
				node.deleteFromModel();
			} else {
				logger.error("Fishy: node is not netnode: " + object);
			}
		}

	}

	/**
     * 
     * @see deleteNodes
     */

	private void deleteEdges(DefaultGraphModel oldModel) {
		List edges = oldModel.getEdges();
		int size = edges.size();
		for (int i = 0; i < size; i++) {
			Object object = edges.get(0);
			if (object instanceof NetEdge) {
				NetEdge edge = (NetEdge) object;
				edge.deleteFromModel();
			} else {
				logger.error("Fishy: edge is not NetEdge: " + object);
			}
		}
	}

	private JPanel getActivityPanel(Activity a) {

		/**
         * An activity can also have conditions, or?
         * 
         */
		GridLayout layout = new GridLayout(0, 1);
		JPanel activityPane = new JPanel(layout);
		activityPane.setBorder(BorderFactory.createTitledBorder("Create an activity"));
		activityPane.add(ActivityUtils.getNameParameter(a).getComponent());
		for (ActivityParameter param : a.getParams()) {
			if (param instanceof AbstractTaskViewParameter) {
				TaskViewParameter taskParam = (TaskViewParameter) param;
				activityPane.add(taskParam.getComponent());
			}
		}
		return activityPane;

	}

	public Rectangle getActivityArea(Activity a) {
		return TaskView.getActivityArea(graph, a);
	}

	public static Rectangle getActivityArea(JGraph graph, Activity a) {
		LayerPerspective layer = (LayerPerspective) graph.getEditor().getLayerManager().getActiveLayer();
		List figNodes = layer.getContentsNoEdges();
		Rectangle bounds = null;
		for (Object object : figNodes) {
			if (object instanceof ActivityFigNode) {
				ActivityFigNode figNode = (ActivityFigNode) object;
				if (figNode.getActivityNode().getActivity().equals(a)) {
					bounds = figNode.getBounds();
					break;
				}
			} else {
				Logger.getLogger(TaskView.class).error("Wrong type of node: " + object.getClass());
			}
		}
		return bounds;
	}

	private void addFileMenu() {
		MenuBar mb = mainWindow.getMenuBar();
		Menu fileMenu = new Menu("File");
		mb.add(fileMenu);
		MenuItem saveItem = new MenuItem("Save");
		saveItem.setShortcut(new MenuShortcut(KeyEvent.VK_S));
		saveItem.addActionListener(new SaveItem());
		fileMenu.add(saveItem);
		MenuItem openItem = new MenuItem("Open");
		openItem.setShortcut(new MenuShortcut(KeyEvent.VK_O));
		fileMenu.add(openItem);
		openItem.addActionListener(new OpenItem());

		MenuItem resetItem = new MenuItem("Reset graph");
		fileMenu.add(resetItem);
		resetItem.addActionListener(new ResetItem());

	}

	private void addActivityMenu() {
		MenuBar mb = mainWindow.getMenuBar();
		Menu activityMenu = new Menu("Activity menu");
		mb.add(activityMenu);
		for (Class activityClass : ReferenceHolder.activityClasses) {
			activityMenu.add(new ActivityAddItem(this, activityClass, "Add " + activityClass.getSimpleName()));
		}
	}

	private void createNode(Class activityClass) {
		GraphModel model = graph.getGraphModel();
		if (model instanceof DefaultGraphModel) {
			DefaultGraphModel defaultGraphModel = (DefaultGraphModel) model;
			ActivityNode activityNode = new ActivityNode(activityClass);
			activityNode.initialize(null);
			defaultGraphModel.addNode(activityNode);
		} else {
			logger.error("Wrong graph model used!");
		}
	}

	/**
     * These methods are replacements for JGraph.setModel which doesn't seem to
     * behave as expected
     * 
     */
	private void addEdges(DefaultGraphModel model, DefaultGraphModel oldModel) {
		List edges = model.getEdges();
		for (Object object : edges) {
			if (object instanceof NetEdge) {
				NetEdge edge = (NetEdge) object;
				oldModel.addEdge(edge);
			} else {
				logger.error("Edge to add is not NetEdge: " + object);
			}
		}
	}

	private void addNodes(DefaultGraphModel model, DefaultGraphModel oldModel) {
		List nodes = model.getNodes();
		for (Object object : nodes) {
			if (object instanceof ActivityNode) {
				ActivityNode node = (ActivityNode) object;
				oldModel.addNode(node);
				node.postLoad();
			} else {
				Logger.getLogger(getClass()).error("Node to add is not an ActivityNode: " + object);
			}
		}
	}

	/**
     * @param args
     */
	public static void main(String[] args) {
		new TaskView(new JFrame());
	}

	public List<Activity> getEvaluationActivities() {
		return Utils.toVector(ReferenceHolder.allActivities);
	}

	/**
     * 
     * Notify critics that the view has been updated.
     * 
     * @see critiquer.gui.graph.GraphRenderingListener#graphRendered()
     */
	public void graphRendered() {
		updateView((Graphics2D) getGraphics());
		/**
         * The editor pane may stretch below the surface of the graph so we may
         * interfere with the south pane when drawing around activities,
         * therefore it needs to be redrawn. This could be fixed by letting the
         * activity areas of this panel be relative to the graph and sending
         * updateView the graphics context of the graph instead of the view,
         * just as in the time view. This has the disadvantage that we cannot
         * draw between activities and their parameters however..
         * 
         */
		southPane.repaint();
	}

}
