/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.gui;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.util.Calendar;
import java.util.GregorianCalendar;

import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotRenderingInfo;
import org.jfree.data.gantt.TaskSeriesCollection;

public class TimeLinePlot extends CategoryPlot {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private View view;
	private DateAxis dateAxis;
	private Rectangle2D dataArea;
	/**
	 * currentTime is a coordinate along the x axis of the time line, which can be converted into a date by the locationToDate method.
	 */
	private Double currentTime = Double.NaN;
	
	
	public TimeLinePlot(TaskSeriesCollection tasks, CategoryAxis categoryAxis, DateAxis dateAxis, TimeLineRenderer renderer) {
		super(tasks, categoryAxis, dateAxis, renderer);
		this.dateAxis=dateAxis;
	}

	/**
	 * Render the plot as usual, but also notify the visualization constraints
	 */
	@Override
	public boolean render(Graphics2D g2, Rectangle2D dataArea, int index, PlotRenderingInfo info) {
		boolean val = super.render(g2, dataArea, index, info);
		/**
		 * If currentTime has some sensible value, draw the current time as a vertical line 
		 */
		if (! currentTime.isNaN()) {
			Color oldColor = g2.getColor();
			g2.setColor(Color.RED);
			g2.drawLine(currentTime.intValue(), (int) dataArea.getY(), currentTime.intValue(), (int) dataArea.getMaxY());
			g2.setColor(oldColor);
		}
		
		this.dataArea = dataArea; 
		view.updateView(g2);
		return val;
	}


	public void setView(View view) {
		this.view=view;
	}
	/**
	 * Translate an x coordinate in the coordinate space of the chart panel to a date on the time line.
	 * 
	 * @param x the coordinate in the chart panel coordinate space.
	 * @return a Calendar object representing the date.
	 */
	public Calendar locationToDate(double x) {
		Calendar cal = new GregorianCalendar();
		cal.setTimeInMillis((long) dateAxis.java2DToValue(x, dataArea, getRangeAxisEdge()));
		return cal;
	}

	/**
	 * @return Returns the currentTime.
	 */
	public Calendar getCurrentTime() {
		if (currentTime.isNaN()) {
			currentTime=dataArea.getX();
		}
		return locationToDate(currentTime);
	}

	/**
	 * @param currentTime The currentTime to set.
	 */
	public void setCurrentTime(Double currentTime) {
		this.currentTime = currentTime;
	}

	/**
	 * @return Returns the dataArea.
	 */
	public Rectangle2D getDataArea() {
		return dataArea;
	}
	
}
