/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.gui;

import java.awt.Graphics2D;
import java.awt.Shape;
import java.util.List;

import javax.swing.JPanel;

import critiquer.activities.Activity;



/**
 * 
 * All plan views should implement this interface..
 * 
 * @author olale
 *
 */
public interface View {

	/**
	 * A view should render activities as either components or as Java 2D Shapes. We only use rectangles currently.
	 * If the view does not support an operation it may throw an "UnsupportedOperationException"
	 * */
	public Shape getActivityArea(Activity activity);	
	public List<Activity> getEvaluationActivities();
	public void updateView(Graphics2D g2);
	/**
	 * 
	 * All views should let constraints show information as components in a special panel
	 * 
	 * @return the panel where the constraints may render components that display some critiquing information
	 */
	public JPanel getConstraintPanel();
}
