/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.gui.graph;

import java.awt.Color;

import org.tigris.gef.presentation.FigCircle;
import org.tigris.gef.presentation.FigNode;
import org.tigris.gef.presentation.FigRect;
import org.tigris.gef.presentation.FigText;

import critiquer.activities.AbstractParamChangedListener;
import critiquer.activities.Activity;
import critiquer.activities.parameters.CommanderParameter;
import critiquer.activities.parameters.NameParameter;
import critiquer.activities.parameters.Parameter;



/**
 * Visual representation of an activity in the graph view
 * 
 * @author olale
 *
 */
public class ActivityFigNode extends FigNode {

	/**
	 * 
	 */
	private static final long	serialVersionUID	= 1L;

	private FigRect		figNodeFrame;

	private FigCircle	parentPortFig;

	private FigCircle	childPortFig;

	private FigRect		previousPortFig;

	private FigRect		nextPortFig;

	private FigText		descriptionFig;

	public ActivityFigNode(ActivityNode node) {
		super(node);
        //		 Bind ports to objects used by this graphical representation
		ActivityNode activityNode = (ActivityNode) node;
		figNodeFrame = new FigRect(-50,-50,100,100,true);
		addFig(figNodeFrame);
		parentPortFig = new GraphFigCircle( -10, -50, 20, 20, Color.black, Color.blue);
		addFig(parentPortFig);
		childPortFig = new GraphFigCircle( -10,  30, 20, 20, Color.black, Color.blue);
		addFig(childPortFig);
		previousPortFig = new FigRect(-60,  -10, 20, 20, Color.black, Color.green);
		addFig(previousPortFig);
		nextPortFig = new FigRect( 40,  -10, 20, 20, Color.black, Color.green);
		addFig(nextPortFig);
		descriptionFig = new FigText( -40,  -25, 90, 30);
		descriptionFig.setLineWidth(0);
		descriptionFig.setFontFamily("Tahoma");
		descriptionFig.setFontSize(11);
		descriptionFig.setJustification(FigText.JUSTIFY_LEFT);
		descriptionFig.setLeftMargin(5);
		descriptionFig.setRightMargin(40);
		descriptionFig.setWordWrap(true);
		descriptionFig.setFilled(false);
		updateDescriptionText();
		addFig(descriptionFig);
		
		bindPort(activityNode.getParentPort(), parentPortFig);
		bindPort(activityNode.getChildPort(), childPortFig);
		bindPort(activityNode.getPreviousPort(), previousPortFig);
		bindPort(activityNode.getNextPort(), nextPortFig);
		setResizable(false);
		
		node.getActivity().addPrivateUpdateListener(new AbstractParamChangedListener() {

			/**
			 * 
			 */
			private static final long serialVersionUID = 1L;

			@Override
			public <T> void paramChanged(Activity activity, Parameter<T> p) {
				if (p instanceof CommanderParameter) {
					updateHighlight(activity);
					
				} else if (p instanceof NameParameter) {
					updateDescriptionText();
				}
				getLayer().damageAll();
			}

			
			
		});
		updateHighlight(node.getActivity());
		
	}
	
	private void updateHighlight(Activity activity) {
		if (activity.isAgentAssignable()) {
			figNodeFrame.setFilled(true);
			figNodeFrame.setFillColor(Color.LIGHT_GRAY);
		} else {
			figNodeFrame.setFillColor(Color.WHITE);
			//figNodeFrame.setFilled(false);
		}
	}
	
	private void updateDescriptionText() {
		descriptionFig.setText(getActivityNode().getActivity().toString());
	}

	public ActivityNode getActivityNode(){
		return (ActivityNode) getOwner();
	}


	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return getActivityNode().getActivity().toString();
	}

	
	
}
