/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.gui.graph;

import java.io.PrintStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.lang.reflect.Constructor;
import java.util.Hashtable;

import org.apache.log4j.Logger;
import org.tigris.gef.base.Layer;
import org.tigris.gef.graph.presentation.NetNode;
import org.tigris.gef.presentation.FigNode;

import critiquer.activities.Activity;
import critiquer.activities.ActivityUtils;


public class ActivityNode extends NetNode {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	public transient static PrintStream debug = System.err;

	private Activity activity;

	private ParentPort parentPort;

	private ChildPort childPort;

	private PreviousPort previousPort;

	private NextPort nextPort;

	private transient ActivityFigNode fn;

	private transient Logger logger = Logger.getLogger(getClass());

	public ActivityNode(Class activityClass) {
		if (Activity.class.isAssignableFrom(activityClass)) {
			try {
				Constructor constructor = activityClass
						.getConstructor(new Class[] { Activity.class });
				activity = (Activity) constructor
						.newInstance(new Object[] { null });
			} catch (Exception e) {
				logger.error(e);
				StringWriter stackTraceWriter = new StringWriter();
				e.printStackTrace(new PrintWriter(stackTraceWriter));
				logger.error(stackTraceWriter.getBuffer());
			}
		} else {
			logger.error("Trying to initialize non-activity class: "
					+ activityClass.getSimpleName());
		}

	}

	@Override
	public void initialize(Hashtable args) {
		parentPort = new ParentPort(this);
		childPort = new ChildPort(this);
		previousPort = new PreviousPort(this);
		nextPort = new NextPort(this);
		addPort(parentPort);
		addPort(childPort);
		addPort(previousPort);
		addPort(nextPort);
	}

	@Override
	public FigNode makePresentation(Layer lay) {
		if (fn == null) {
			fn = new ActivityFigNode(this);
		}
		return fn;
	}

	@Override
	public String getId() {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * @return Returns the childPort.
	 */
	public ChildPort getChildPort() {
		return childPort;
	}

	/**
	 * @return Returns the nextPort.
	 */
	public NextPort getNextPort() {
		return nextPort;
	}

	/**
	 * @return Returns the parentPort.
	 */
	public ParentPort getParentPort() {
		return parentPort;
	}

	/**
	 * @return Returns the previousPort.
	 */
	public PreviousPort getPreviousPort() {
		return previousPort;
	}

	/**
	 * @return Returns the activity.
	 */
	public Activity getActivity() {
		return activity;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return activity.toString();
	}

	/**
	 * Restore necessary properties after reading this node from a file
	 */
	public void postLoad() {
		// Restore the graphical representation
		fn = new ActivityFigNode(this);
		getActivity().postLoad();
	}

	public void preSave() {
		getActivity().preSave();
	}

	/**
	 * 
	 * Remove the activity, along with any constraints related to it and notify listeners it has been removed.
	 * @see org.tigris.gef.graph.presentation.NetNode#deleteFromModel()
	 */
	@Override
	public void deleteFromModel() {
		super.deleteFromModel();
		ActivityUtils.removeConstraintsForActivity(getActivity());
		getActivity().signalActivityRemoved();
	}
	
}
