/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.gui.graph;

import org.apache.log4j.Logger;
import org.tigris.gef.base.Layer;
import org.tigris.gef.graph.GraphModel;
import org.tigris.gef.graph.presentation.NetEdge;
import org.tigris.gef.presentation.ArrowHeadTriangle;
import org.tigris.gef.presentation.FigEdge;

import critiquer.activities.Activity;


public class InheritanceEdge extends NetEdge {

	/**
     * 
     */
	private static final long  serialVersionUID = 1L;

	private InheritanceFigEdge edge;

	private ChildPort		  childPort;

	private ParentPort		 parentPort;

	private Activity		   child;

	private Activity		   parent;

	private transient Logger logger = Logger.getLogger(getClass());

	public InheritanceEdge() {
		super();
		// TODO Auto-generated constructor stub
	}

	@Override
	public FigEdge makePresentation(Layer lay) {
		if (edge == null) {
			edge = new InheritanceFigEdge();
			if (getSourcePort() instanceof ChildPort) {
				// Drawing edge from child to parent
				edge.setDestArrowHead(new ArrowHeadTriangle());

			} else if (getSourcePort() instanceof ParentPort) {
				// Drawing edge from parent to child
				edge.setSourceArrowHead(new ArrowHeadTriangle());

			}
		}
		return edge;
	}

	/**
     * 
     * The parentPort is the port that connects TO a parent and the childPort is
     * the port that connects TO the children.
     * 
     * That means that a parentPort is located on an ActivityFigNode which
     * represents an activity that is supposed to be the child of the activity
     * associated with the childPort.
     * 
     */
	@Override
	public boolean connect(GraphModel gm, Object srcPort, Object destPort) {
		boolean connect = super.connect(gm, srcPort, destPort);
		childPort = null;
		parentPort = null;
		if (srcPort instanceof ChildPort && destPort instanceof ParentPort) {
			childPort = (ChildPort) srcPort;
			parentPort = (ParentPort) destPort;
		} else if (destPort instanceof ChildPort && srcPort instanceof ParentPort) {
			childPort = (ChildPort) destPort;
			parentPort = (ParentPort) srcPort;
		} else {
			logger.error("Cannot connect " + srcPort.getClass().getSimpleName()
							   + " and "
							   + destPort.getClass().getSimpleName()
							   + " with an InheritanceEdge");
			connect = false;
		}
		if (connect) {
			child = parentPort.getActivityNode().getActivity();
			parent = childPort.getActivityNode().getActivity();
			child.initParentRelationship(parent);
		}

		return connect;

	}

	/**
     * @see critiquer.gui.graph.InheritanceEdge#connect(org.tigris.gef.graph.GraphModel,java.lang.Object,java.lang.Object)
     *      for an explanation on how the parentPort should be interpreted.
     */
	@Override
	public void deleteFromModel() {
		super.deleteFromModel();
		parentPort.getActivityNode().getActivity().endParentRelationship();
	}

	@Override
	public String getId() {
		// TODO Auto-generated method stub
		return null;
	}

	/**
     * @return Returns the child.
     */
	public Activity getChild() {
		return child;
	}

	/**
     * @return Returns the parent.
     */
	public Activity getParent() {
		return parent;
	}

}
