/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.gui.graph;

import org.apache.log4j.Logger;
import org.tigris.gef.base.Layer;
import org.tigris.gef.graph.GraphModel;
import org.tigris.gef.graph.presentation.NetEdge;
import org.tigris.gef.presentation.FigEdge;

import critiquer.activities.Activity;


public class OrderingEdge extends NetEdge {

	/**
     * 
     */
	private static final long serialVersionUID = 1L;

	private OrderingFigEdge   figEdge;

	private PreviousPort	  previousPort	 = null;

	private NextPort		  nextPort		 = null;

	private Activity		  beforeActivity;

	private Activity		  afterActivity;

	public OrderingEdge() {
		super();
		// TODO Auto-generated constructor stub
	}

	@Override
	public FigEdge makePresentation(Layer lay) {
		if (figEdge == null) {
			figEdge = new OrderingFigEdge();
			figEdge.setDashed(true);

		}
		return figEdge;
	}

	/**
     * 
     * <p>
     * Just as in the case with inheritance ports, the nextPort denotes the port
     * connecting TO the next activity.
     * </p>
     * <p>
     * That means that the nextPort is connected to a Fig that represents the
     * activitiy that should come <b>before</b>.
     * </p>
     * 
     * 
     */
	@Override
	public boolean connect(GraphModel gm, Object srcPort, Object destPort) {
		boolean connect = super.connect(gm, srcPort, destPort);
		if (srcPort instanceof PreviousPort && destPort instanceof NextPort) {
			previousPort = (PreviousPort) srcPort;
			nextPort = (NextPort) destPort;
		} else if (destPort instanceof PreviousPort && srcPort instanceof NextPort) {
			previousPort = (PreviousPort) destPort;
			nextPort = (NextPort) srcPort;
		} else {
			Logger.getLogger(getClass()).error("Cannot connect " + srcPort.getClass().getSimpleName()
							   + " and "
							   + destPort.getClass().getSimpleName()
							   + " with OrderingEdge");
			connect = false;
		}
		if (connect) {
			beforeActivity = nextPort.getActivityNode().getActivity();
			afterActivity = previousPort.getActivityNode().getActivity();
			beforeActivity.order(afterActivity);
		}
		return connect;
	}

	/**
     * 
     * @see critiquer.gui.graph.OrderingEdge#connect(org.tigris.gef.graph.GraphModel,java.lang.Object,java.lang.Object)
     *      for information on what the ports denote
     * 
     */
	@Override
	public void deleteFromModel() {
		super.deleteFromModel();
		nextPort.getActivityNode().getActivity().removeOrder(previousPort.getActivityNode()
																						.getActivity());
	}

	@Override
	public String getId() {
		// TODO Auto-generated method stub
		return null;
	}

	/**
     * @return Returns the afterActivity.
     */
	public Activity getAfterActivity() {
		return afterActivity;
	}

	/**
     * @return Returns the beforeActivity.
     */
	public Activity getBeforeActivity() {
		return beforeActivity;
	}

}
