/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.misc;

import java.util.ArrayList;
import java.util.HashSet;

import javax.swing.JFrame;

import org.tigris.gef.graph.presentation.DefaultGraphModel;

import critiquer.activities.Activity;
import critiquer.activities.ActivityFactory;
import critiquer.activities.parameters.Commanders;
import critiquer.communication.Communicator;
import critiquer.constraints.ActivityStructureCritic;
import critiquer.constraints.AgentMutex;
import critiquer.constraints.ConstraintFactory;
import critiquer.constraints.FuelLowCritic;
import critiquer.constraints.NonSuitableAgent;
import critiquer.constraints.ResourceUsageCritic;
import critiquer.constraints.StandardConstraint;
import critiquer.constraints.TimeCalculator;
import critiquer.constraints.TimeParameterOrdering;
import critiquer.constraints.TooLongTime;
import critiquer.gui.CriticTextArea;
import critiquer.gui.GeoView;
import critiquer.gui.ResourceView;
import critiquer.gui.TimeView;
import critiquer.resources.AgentFactory;
import critiquer.resources.OrganizationModel;
import critiquer.scenarios.standard.CorrectOrdering;
import critiquer.scenarios.standard.StandardActivityFactory;
import critiquer.scenarios.standard.StandardAgentFactory;

/**
 * 
 * <p>
 * If it makes sense to have global references to something, then put it here.
 * This could be references to activity classes, constraint classes or such
 * like.
 * </p>
 */

public class ReferenceHolder {

	/**
	 * Should we use hash patterns instead of colors?
	 */
	public static Boolean										monochromeDisplay   = true;

	/**
	 * 
	 * The scenario specific factories that create a list of activity classes as
	 * well as a resource structure as required by the
	 * <code>OrganizationModel</code>.
	 * 
	 * TODO: Change these by creating a scenario selection possibility
	 */
	public static ActivityFactory								activityFactory	 = new StandardActivityFactory();

	public static AgentFactory								   agentFactory		= new StandardAgentFactory();

	/**
	 * The activity classes used in the current scenario. These are created by
	 * the <code>activityFactory</code>.
	 */
	public static ArrayList<Class<? extends Activity>>		   activityClasses;
	static {
		/**
		 * Retrieve a list of activities from the ActivityFactory represented by
		 * the current scenario.
		 */
		activityClasses = activityFactory.getActivityClasses();
	}

	// TODO: Make this parameterized so that constraintClasses can be added
	// dynamically, by pointing to the relevant scenario
	public static ArrayList<Class<? extends StandardConstraint>> constraintClasses   = new ArrayList<Class<? extends StandardConstraint>>();
	static {
		constraintClasses.add(TooLongTime.class);
		constraintClasses.add(NonSuitableAgent.class);
		constraintClasses.add(ResourceUsageCritic.class);
		constraintClasses.add(ActivityStructureCritic.class);
		constraintClasses.add(FuelLowCritic.class);
		constraintClasses.add(AgentMutex.class);
		constraintClasses.add(TimeCalculator.class);
		constraintClasses.add(CorrectOrdering.class);

		/**
		 * 
		 * Add more constraint classes here
		 * 
		 */

		for (Class c : constraintClasses) {
			ConstraintFactory.createSingletonConstraint(c, false);
		}

	}

	public static Communicator								   communicator;

	/**
	 * Static reference to all constraints that order activities in time in some
	 * way.
	 */
	public static ArrayList<TimeParameterOrdering>			   temporalConstraints = new ArrayList<TimeParameterOrdering>();

	/**
	 * Model of all the resources that are available in the organization,
	 * ordered hierarchically according to subsumption. That is, a larger force
	 * element subsumes a smaller iff the smaller element is contained as a part
	 * of the larger.
	 */
	public static OrganizationModel							  organizationModel   = new OrganizationModel();

	/**
	 * 
	 * Static reference to a set of all activities that are created.
	 * 
	 */
	public static HashSet<Activity>							  allActivities	   = new HashSet<Activity>();

	/**
	 * Activity that is currently selected in some view and may therefore be of
	 * particular interest to critics that highlight information on the
	 * currently selected activity.
	 */
	public transient static Activity							 selectedActivity;

	public transient static JFrame							   topFrame;

	public transient static TimeView							 timeView;

	public transient static ResourceView						 resourceView;

	public transient static GeoView							  simulationView;

	private transient static DefaultGraphModel				   graphModel;

	/**
	 * 
	 * To make sure that there is only one graph model used by all views that
	 * want to represent tasks in a graph, we have a singleton graph model
	 * factory here.
	 * 
	 * @return a new graph model if none exists or the one that was created
	 *         during a previous call to this method.
	 */
	public static DefaultGraphModel getGraphModel() {
		if (graphModel == null) {
			graphModel = new DefaultGraphModel();
		}
		return graphModel;
	}

	/**
	 * 
	 * Since the list of commanders should be available to several views that
	 * are not related (the task view and resource view for instance), there is
	 * a reference to it here.
	 * 
	 */
	public static Commanders			   commanders = new Commanders();

	/**
	 * 
	 * Directory containing the map and other image resources
	 * 
	 */
	public static String				   resourceDir;

	/**
	 * Information area that can be used by critics if they want to display
	 * information in textual form
	 */
	public transient static CriticTextArea critiqueInfoArea;

}
