/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.resources;

import java.util.ArrayList;
import java.util.Vector;

import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import critiquer.misc.ReferenceHolder;


/**
 * 
 * The organization model represents the available resources that may be used
 * for this scenario
 * 
 * To change what resources are used, we must change the constructor so that
 * another agent factory is used to produce our set of agents
 * 
 * TODO: Preferably this should be selectable when the planning tool starts, or
 * at least configurable from the command line.
 * 
 * @author olale
 * 
 */
public class OrganizationModel implements TreeModel {

	private Agent								  rootAgent;

	private transient ArrayList<TreeModelListener> listeners = new ArrayList<TreeModelListener>();

	/**
     * Create a default hierarchy
     */

	public OrganizationModel() {
		ReferenceHolder.organizationModel = this;
	}
	/**
	 * Generate an organization structure.
	 *
	 */
	public void generateOrganization() {
		setRootAgent(ReferenceHolder.agentFactory.getAgentStructure());
	}

	/**
     * Get all concrete agents that are not a combination of several agents in a
     * group
     * 
     * @return all agent which have no children 
     */
	public Vector<Agent> getAllAgents() {
		ArrayList<Agent> queue = new ArrayList<Agent>();
		queue.add(rootAgent);
		Vector<Agent> allAgents = new Vector<Agent>();
		while (!queue.isEmpty()) {
			Agent currentAgent = queue.remove(0);
			if (currentAgent.getChildren().isEmpty()) {
				allAgents.add(currentAgent);
			} else {
				queue.addAll(currentAgent.getChildren());
			}

		}

		return allAgents;

	}

	public Object getRoot() {
		return rootAgent;
	}

	public void setRootAgent(Agent rootAgent) {
		this.rootAgent = rootAgent;
	}

	public Object getChild(Object parent, int index) {
		return ((Agent) parent).getChildren().get(index);
	}

	public int getChildCount(Object parent) {
		return ((Agent) parent).getChildren().size();
	}

	public boolean isLeaf(Object node) {
		return ((Agent) node).getChildren().isEmpty();
	}

	public void valueForPathChanged(TreePath path, Object newValue) {
		for (TreeModelListener listener : listeners) {
			listener.treeNodesChanged(new TreeModelEvent(newValue, path));
		}
	}

	public int getIndexOfChild(Object parent, Object child) {
		return ((Agent) parent).getChildren().indexOf(child);
	}

	public void addTreeModelListener(TreeModelListener l) {
		listeners.add(l);
	}

	public void removeTreeModelListener(TreeModelListener l) {
		listeners.remove(l);
	}

}
