/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.resources;

import java.awt.BorderLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.Serializable;
import java.util.Vector;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import critiquer.scenarios.standard.AgentParameterName;


/**
 * This class 
 */

public class SingleSelectionParameter<T> implements Serializable,ParameterInAgent<T> {

	private static final long	serialVersionUID	= 1L;
	private T value;
	private transient JPanel component;
	private transient JComboBox selectionList;
	private AgentParameterName name;
	private transient JLabel nameLabel;
	private Vector<T> range;
	private Agent agent;
	
	private ParameterInAgent.Type type = ParameterInAgent.Type.EXTERNAL;
	
	public SingleSelectionParameter(AgentParameterName name,final Agent agent,Vector<T> range,T defaultObject) {
		this.agent=agent;
		this.name=name;
		value = defaultObject;
		
		if (! range.contains(defaultObject)) {
			range.add(0, defaultObject);
		}
		this.range=range;
	}


	private void initComponents(String name, final Agent agent) {
		component = new JPanel(new BorderLayout());
		nameLabel = new JLabel(name);
		selectionList = new JComboBox(range);
		component.add(nameLabel,BorderLayout.WEST);
		component.add(selectionList,BorderLayout.CENTER);
		selectionList.setSelectedItem(value);
		selectionList.addItemListener(new ItemListener(){

			public void itemStateChanged(ItemEvent e) {
				Object obj = selectionList.getSelectedItem();
				value = (T) obj;
				agent.notifyAgentChangedListeners();
			}
			
		});
	}

	
	public void editEnabled(boolean enabled) {
		selectionList.setEditable(enabled);
	}
	
	/**
	 * @return Returns the component.
	 */
	public JPanel getComponent() {
		if (component==null) {
			initComponents(getName().toString(), agent);
		}
		return component;
	}

	/**
	 * @return Returns the value.
	 */
	public T getValue() {
		return value;
	}

	public AgentParameterName getName() {
		return name;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		boolean eq = false;
		
		if (obj instanceof SingleSelectionParameter) {
			SingleSelectionParameter param = (SingleSelectionParameter) obj;
			if (param.getName().equals(getName())) {
				eq=true;
			}
		}
		
		return eq;
	}


	/**
	 * @return Returns the type.
	 */
	public ParameterInAgent.Type getType() {
		return type;
	}


	/**
	 * @param type The type to set.
	 */
	public void setType(ParameterInAgent.Type type) {
		this.type = type;
	}
	
	
	
}
