/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.scenarios.standard;

import java.awt.Font;
import java.util.HashMap;

import critiquer.resources.Agent;
import critiquer.resources.Suitability;


public class Helicopter extends Agent {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;


	public Helicopter(String name) {
		super(name);
		init();
	}

	
	/**
	 * @param parent
	 * @param name
	 */
	public Helicopter(Agent parent, String name) {
		super(parent, name);
		init();
	}


	private void init() {
		// We need only model which tasks an agent is unfit to perform
		HashMap<Class, Suitability> suitabilityMap = new HashMap<Class, Suitability>();
		suitabilityMap.put(DispatchActivity.class, Suitability.POSSIBLE);
		suitabilityMap.put(FirstAidActivity.class, Suitability.UNFIT);
		suitabilityMap.put(RoadBlockActivity.class, Suitability.UNFIT);
		Agent.fitnessMap.put(Helicopter.class, suitabilityMap);
		
		StandardAgentParameterFactory.createFuelParameters(this,110,15);

		StandardAgentParameterFactory.createSpeedParameter(this,200L);
		
		StandardAgentParameterFactory.createFirstAidParameters(this, 20L, 2);
		
		StandardAgentParameterFactory.createTransportParameter(this,4);
		setPictureFromName("helicopter.jpg");
		initPanel();
	}


	/**
	 * @see critiquer.resources.Agent#getSymbol()
	 */
	@Override
	public Character getSymbol() {
		return '\uF027';
	}

	/**
	 * 
	 * @return the unicode font used to render the symbol String
	 */
	@Override
	public Font getSymbolFont() {
		return new Font("MapSym-NU-Air-APP6a",Font.PLAIN,48);
	}

}
