/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.scenarios.standard;

import java.util.HashSet;

import org.apache.log4j.Logger;

import critiquer.activities.Activity;
import critiquer.activities.ActivityUtils;
import critiquer.activities.parameters.LocationParameter;
import critiquer.constraints.StructureCritic;
import critiquer.resources.Agent;



public class RoadBlockActivity extends Activity {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private static int numActivities;

	private static StructureCritic agentCritic = StandardActivityFactory.createAgentAvailableCritic(RoadBlockActivity.class);

	public RoadBlockActivity(Activity parent) {
		super("Block roads", parent);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see critiquer.activities.Activity#addDefaultParams()
	 */
	@Override
	public void addDefaultParams() {
		super.addDefaultParams();
		addParam(new LocationParameter(LocationParameter.Type.STATIC_LOC,"road block location", this));
	}

	/**
	 * It takes the amount of time of the fastest agent to perform this
	 * operation.
	 */
	@Override
	public long simulate() {
		long timeToComplete = Long.MAX_VALUE;
		// time in minutes
		HashSet<Agent> agentsInActivity = ActivityUtils.getAgentsInActivity(this);
		if (!agentsInActivity.isEmpty()) {
			for (Agent agent : agentsInActivity) {
				try {
					timeToComplete = Math.min(timeToComplete, agent.perform(RoadBlockActivity.class, this, null));
					/**
					 * If an agent is unsuitable for a certain type of operation, we
					 * catch that exception here
					 */
				} catch (UnsupportedOperationException e) {
					Logger.getLogger(getClass()).error(e.getMessage());
				}
			}
		} else {
			timeToComplete=0;
		}
		 
		return timeToComplete;

	}



	@Override
	public int getNumActivities() {
		return RoadBlockActivity.numActivities++;
	}


	/* (non-Javadoc)
	 * @see critiquer.activities.Activity#canBeSimulated()
	 */
	@Override
	public boolean canBeSimulated() {
		return agentCritic.isConsistent(this);
	}

}
