/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.simulation;

import java.awt.Graphics2D;

import critiquer.activities.Activity;
import critiquer.gui.GeoView;
import critiquer.resources.Agent;


/**
 * 
 * <p>
 * When performing a simulation, each agent uses a state that represents the
 * current state of the agent at the time indicated by the simulation
 * environment.
 * </p>
 * 
 * <p>
 * There is also a special breed of constraints (SimulationCritic) that react to
 * simulation changes and display critiquing information during the simulation.
 * </p>
 * 
 * <p>
 * These constraints as well as agent states share a common interface
 * <code>SimulationElement</code> that the simulation engine uses when
 * initializing the simulaiton and stepping forward and backward in time
 * </p>
 * 
 * @author olale
 * 
 */
public interface AgentSimulationState extends SimulationElement {
	/**
     * Enter property values to be used during the simulation
     * 
     * @param key identifying this property
     * @param value the value
     */
	public void setProperty(String key,Object value);

	/**
     * 
     * There may be a number of properties associated with a state of an agent
     * during a simulation. This method returns one based on the name and type
     * of the property. Properties may be fuel level, current radar coverage or
     * other information.
     * 
     * @param name
     *            of the property
     * @return the value of property <code>name</code>, <code>null</code>
     *         if not available.
     */

	public Object getProperty(String name);

	/**
	 * 
	 * It can be useful to simulate only isolated activities, in which case this method can be used.
	 * 
	 * @param activity
	 */
	public void consumeActivity(Activity activity);
	
	/**
     * 
     * Render a visual representation of this agent state in the simulation
     * view. This is one of the main purposes of a simulation: To see what the
     * allocated resources will do over time during the simulation.
     * 
     * @param view
     * @param g2
     */
	public void renderState(GeoView view, Graphics2D g2);

	public Agent getAgent();
}
