/**
 * planningtool - A Planning Tool with Critiquing Support.
 * 
 * Copyright (C) 2006 olale

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 * Contact information: 
 * E-mail: olale@ida.liu.se
 *         olale@lysator.liu.se
 */
package critiquer.simulation;

import java.io.Serializable;
import java.util.Calendar;

public interface SimulationElement extends Serializable {

	/**
	 * 
	 * this method is called when the simulation is initiated.
	 * 
	 */
	public void initSimulation(SimulationEngine engine);

	
	/**
	 * Called by the simulation engine when the time is increased in the
	 * simulation. The time amount must be positive and is measured in minutes.
	 * 
	 * @param step
	 *            in minutes forward
	 */
	public void stepForward(int step);

	/**
	 * Called by the simulation engine when the time is decreased in the
	 * simulation. The time amount must be negative and is measured in minutes.
	 * 
	 * @param step
	 *            in minutes backwards
	 */
	public void stepBackward(int step);
	
	/**
	 * Called by the simulation engine when it is useful to step to a fix point in time instead of incrementally 
	 * 
	 * @param time
	 */
	public void stepTo(Calendar time);

}